#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <string>
#include <algorithm>
#include <stack>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <cassert>
using namespace std;

#define rep(i,a,b) for(__typeof(b) i=a; i<(b); ++i)

const int MAX_LINES = 1000;
const int MAX_OPERATIONS = 10000;

const long long MINVAL = -(1LL << 31LL);
const long long MAXVAL = (1LL << 31LL) - 1LL;

void wrong(const string &msg) {
    cout << "Wrong answer: " << msg << endl;
    exit(1);
}

vector<pair<string, long long>> parse(const string &line) {
    if (line == "") {
        wrong("Line can't be empty");
    }
    vector<pair<string, long long>> commands;
    int start = 0;
    while (true) {
        size_t end = line.find(';', start);
        string command = line.substr(start, end - start);
        string prefix = command.substr(0, 4);

        if (command == "") {
            wrong("Extra semicolon");
        }
        if (command[0] == ' ' || command.back() == ' ') {
            wrong("Extra whitespace");
        }
        if (command == "input" || command == "pop" || command == "print" ||
            command == "dup" || command == "+" || command == "-" ||
            command == "*" || command == "/") {
            commands.push_back(make_pair(command, -1));
        } else if (prefix == "push" || prefix == "jump") {
            long long number;
            if (command.size() < 6 || sscanf(command.substr(5).c_str(), "%lld", &number) != 1) {
                wrong("Failed to parse a number");
            }
            char buf[80];
            snprintf(buf, 80, "%s %lld", prefix.c_str(), number);
            if (command != string(buf)) {
                wrong("Failed to parse a number");
            }
            commands.push_back(make_pair(prefix, number));
        } else {
            wrong("Unknown command or other syntax error");
        }

        if (end == string::npos) break;
        start = end + 1;
    }
    return commands;
}

pair<string, long long> evaluate(const vector<string>& program, long long input) {
    stack<long long> state;
    int reads = 0;
    vector<long long> prints;
    int operations = 0;
    int cur = 0;
    while (cur != (int)program.size()) {
        vector<pair<string,long long>> commands = parse(program[cur]);
        for (const pair<string,long long>& command : commands) {
            operations++;
            if (operations > MAX_OPERATIONS) return {"Infinite loop", -1};

            if (command.first == "input") {
                ++reads;
                if (reads > 1) return {"Executed input multiple times", -1};
                state.push(input);
            } else if (command.first == "pop") {
                if (state.empty()) return {"Popping an empty stack", -1};
                state.pop();
            } else if (command.first == "print") {
                if (state.empty()) return {"Printing an empty stack", -1};
                prints.push_back(state.top());
                state.pop();
            } else if (command.first == "dup") {
                if (state.empty()) return {"Duplicating an empty stack", -1};
                state.push(state.top());
            } else if (command.first == "push") {
                state.push(command.second);
            } else if (command.first == "jump") {
                if (command.second < 0 || command.second > (long long)program.size()) return {"Invalid jump", -1};
                cur = command.second - 1; // Odpocitaj 1 lebo neskor jednotku priratame
                break; // Ukonci vykonavanie riadku
            } else {
                // All operations are binary
                if (state.size() < 2) return {"Binary operation needs 2 arguments", -1};
                long long a = state.top(); state.pop();
                long long b = state.top(); state.pop();
                if (command.first == "+") state.push(a + b);
                else if (command.first == "-") state.push(a - b);
                else if (command.first == "*") state.push(a * b);
                else if (command.first == "/") {
                    if (b == 0) return {"Division by 0", -1};
                    state.push(a / b);
                } else {
                    return {"Unknown command", -1};
                }
            }
            if (!state.empty() && (state.top() < MINVAL || state.top() > MAXVAL)) {
                return {"Integer overflow", -1};
            }
        }
        cur++;
    }

    if (prints.size() == 0) return {"Program didn't print anything", -1};
    if (prints.size() == 1) return {"", prints[0]};
    return {"Too many numbers printed", -1};
}

int check(const vector<string>& program, long long input, long long output) {
    auto result = evaluate(program, input);
    if (result.first != "") {
        cout << "Wrong answer: " << result.first << endl;
        return 1;
    }
    if (result.second != output) {
        cout << "Wrong answer: Wrong output of the program" << endl;
        return 1;
    }
    return 0;
}

int main(int argc, char* argv[]) {
    assert(argc == 3);
    string difficulty = argv[1];
    assert(difficulty == "easy" || difficulty == "hard");
    int max_on_line = difficulty == "easy" ? 1000 : 2;

    ifstream program(argv[2]);
    assert(program);
    vector<string> lines;
    string line;
    while (getline(program, line)) {
        if (!line.empty() && line.back() == '\r') {
            line.pop_back();
        }
        if ((int)parse(line).size() > max_on_line) {
            wrong("Too many commands on one line");
        }
        lines.push_back(line);
    }
    if (lines.size() > MAX_LINES) wrong("Too many lines");
    vector<string> reversed = lines;
    reverse(reversed.begin(), reversed.end());
    vector<string> sorted = lines;
    sort(sorted.begin(), sorted.end());

    vector<long long> inputs = {1, -1, 4, 47, 123, 11110, 22381, 29870, -42, 0, -28654};
    for (long long input : inputs) {
        long long result = check(lines, input, input);
        if (result != 0) return result;

        result = check(reversed, input, input * input);
        if (result != 0) return result;

        result = check(sorted, input, -input);
        if (result != 0) return result;
    }
    cout << "OK" << endl;
}
